package org.norgg.hopesington;

import nape.callbacks.InteractionType;
import nape.dynamics.InteractionGroup;
import nme.display.DisplayObjectContainer;
import nme.geom.Matrix;
import nme.media.Sound;

import nape.geom.Vec2;
import nape.phys.Body;
import nape.phys.BodyType;
import nape.shape.Polygon;

import nme.Assets;

class Arrow extends Projectile {

	public static inline var w = 8;
	public static inline var h = 3;
	
	var ttl = 300;
	public static var fireSounds: Array<Sound> = null;
	
	public function new(parent: DisplayObjectContainer, pos: Vec2, vel: Vec2, group: InteractionGroup, owner: Person ) {
		super(parent, pos, vel, owner);
		
		body = new Body();
		body.type = BodyType.DYNAMIC;
		body.userData.blocks = false;
		body.shapes.add(new Polygon(Polygon.box(w, h)));
		body.position = pos;
		body.align();
		body.velocity = vel;
		body.group = group;
		body.rotation = vel.angle;
		body.isBullet = true;
		body.space = Main.space;

		var mat = new Matrix();
		mat.translate( -w / 2, -h / 2);
		
		graphics.beginBitmapFill(Assets.getBitmapData("img/arrow.png"), mat, false);
		graphics.drawRect( -w / 2, -h / 2, w, h);
		
		if (fireSounds == null) {
			fireSounds = new Array<Sound>();
			for (i in 0...3) {
				fireSounds[i] = Assets.getSound("snd/fire"+i+".wav");
			}
		}
		fireSounds[Std.int(Math.random()*3)].play();
	}
	
	override public function update() {
		super.update();
		
		ttl--;
		if (ttl <= 0) {
			destroy();
		}
		
		//body.torque = Math.sin(body.velocity.angle - body.rotation) * body.velocity.length;
		for (other in body.interactingBodies(InteractionType.COLLISION, 1)) {
			if (other.userData.person != null) {
				other.userData.person.kill(this);
				destroy();
			}
		}
	}
}