package org.norgg.hopesington;

import flash.display.BitmapData;
import haxe.FastList;
import nape.geom.AABB;
import nape.geom.Vec2;
import nape.phys.Body;
import nape.phys.BodyType;
import nape.phys.Material;
import nape.shape.Polygon;
import nape.space.Space;
import nme.Assets;
import nme.display.Sprite;
import nme.events.Event;
import nme.events.KeyboardEvent;
import nme.events.MouseEvent;
import nme.Lib;
import nme.text.TextField;
import nme.text.TextFieldAutoSize;
import nme.text.TextFormat;
import nme.ui.Keyboard;

class Main extends Sprite {
	
	public static var m: Main;
	public static var space: Space;
	public static var players: Array<Player>;
	public static var npcs: FastList<NPC>;
	public static var projectiles: FastList<Projectile>;
	public static var trees: FastList<Tree>;
	public static var goodHouses: FastList<Building>;
	public static var temple: Temple;
	public static var tick = 0;
	public static var ageLabel: TextField;
	public static var yrsLabel: TextField;
	public static var startLabel: TextField;
	
	public static var started = false;
	public static var startedOverlay: Sprite;
	
	public function new() {
		super();
		#if iphone
		Lib.current.stage.addEventListener(Event.RESIZE, init);
		#else
		addEventListener(Event.ADDED_TO_STAGE, init);
		#end
		addEventListener(Event.ENTER_FRAME, update);	
	}

	private function init(e) {
		m = this;
		stage.addEventListener(KeyboardEvent.KEY_DOWN, keyDown);
		stage.addEventListener(KeyboardEvent.KEY_UP, keyUp);
		stage.addEventListener(MouseEvent.CLICK, click);
				
		reset();
		
		started = false;
		startedOverlay = new Sprite();
		startedOverlay.graphics.beginFill(0, 0.7);
		startedOverlay.graphics.drawRect(0, 0, stage.stageWidth, stage.stageHeight);
		addChild(startedOverlay);
		
		if (startLabel != null) removeChild(startLabel);
		startLabel = new TextField();
		addChild(startLabel);
		startLabel.x = stage.stageWidth/2 - 30;
		startLabel.y = stage.stageHeight/2 - 24;
		startLabel.width = 60;
		startLabel.defaultTextFormat = new TextFormat("_sans", 48, 0xffffff, true);
		startLabel.selectable = false;
		startLabel.autoSize = TextFieldAutoSize.CENTER;
		startLabel.text = "click";
	}
	
	public function reset() {
		tick = 0;
		graphics.clear();
		while (numChildren > 0) removeChildAt(numChildren - 1);
		
		if (ageLabel != null) stage.removeChild(ageLabel);
		ageLabel = new TextField();
		stage.addChild(ageLabel);
		ageLabel.x = 4;
		ageLabel.y = 4;
		ageLabel.width = 60;
		ageLabel.defaultTextFormat = new TextFormat("_sans", 48, 0x000000, true);
		ageLabel.selectable = false;
		ageLabel.autoSize = TextFieldAutoSize.RIGHT;
		ageLabel.text = "10";
		
		if (yrsLabel != null) stage.removeChild(yrsLabel);
		yrsLabel = new TextField();
		stage.addChild(yrsLabel);
		yrsLabel.x = 62;
		yrsLabel.y = 4;
		yrsLabel.width = 30;
		yrsLabel.defaultTextFormat = new TextFormat("_sans", 24, 0x000000, true);
		yrsLabel.selectable = false;
		yrsLabel.autoSize = TextFieldAutoSize.LEFT;
		yrsLabel.text = "yrs";
		yrsLabel.background = false;

		
		space = new Space(new Vec2(0, 0));
		npcs = new FastList<NPC>();
		projectiles = new FastList<Projectile>();
		trees = new FastList<Tree>();
		goodHouses = new FastList<Building>();
		
		//walls
		var walls = new Body();
		walls.shapes.add(new Polygon(Polygon.rect( -10, 0, 10, stage.stageHeight)));
		walls.shapes.add(new Polygon(Polygon.rect(stage.stageWidth, 0, 10, stage.stageHeight)));
		walls.shapes.add(new Polygon(Polygon.rect(0, -10, stage.stageWidth, 10)));
		walls.shapes.add(new Polygon(Polygon.rect(0, stage.stageHeight, stage.stageWidth, 10)));
		walls.setShapeMaterials(new Material(0.5));
		walls.type = BodyType.STATIC;
		walls.space = space;
		
		for (i in 0...70) addTree();
		for (i in 0...4)  addGoodHouse(true);
		temple = null;
		addTemple();
		players = [new Player(this, goodHouses.first().body.position)];
		
		graphics.beginBitmapFill(Assets.getBitmapData("img/bg.png"));
		graphics.drawRect(0, 0, stage.stageWidth, stage.stageHeight);
		
		update();
	}
	
	public function addTree() {
		for (i in 0...10) {
			var pos = new Vec2(
				25 + Math.random() * (stage.stageWidth - 50), 
				25 + Math.random() * (stage.stageHeight - 50)
			);
			var checkAABB = new AABB(pos.x, pos.y, Tree.w, Tree.h);
			var bodies = Main.space.bodiesInAABB(checkAABB);
			if (bodies.length > 0) {
				continue;
			} else { 
				trees.add(new Tree(this, pos));
				break;
			}
		}
	}
	
	public function addNPC(pos: Vec2) {
		var npc = new NPC(this, pos);
		npcs.add(npc);
		npc.age = 500 + Std.int(Math.random() * 1000);
	}
	
	public function addGoodHouse(addNPCs = false) {
		for (i in 0...10) {
			var pos = new Vec2(
				50 + Math.random() * (stage.stageWidth - 100), 
				50 + Math.random() * (stage.stageHeight - 100)
			);
			var checkAABB = new AABB(pos.x, pos.y, GoodHouse.baseWidth, GoodHouse.baseHeight);
			var bodies = Main.space.bodiesInAABB(checkAABB);
			if (bodies.length > 0) {
				continue;
			} else {
				goodHouses.add(new GoodHouse(this, pos));
				
				if (addNPCs) {
					addNPC(pos);
					addNPC(pos);
				}
				
				break;
			}
		}
	}
	
	public function addTemple() {
		if (temple != null) return;
		for (i in 0...20) {
			var pos = new Vec2(
				50 + Math.random() * (stage.stageWidth - 100), 
				stage.stageHeight/2 + Math.random() * (stage.stageHeight/2 - 50)
			);
			var checkAABB = new AABB(pos.x, pos.y, Temple.baseWidth, Temple.baseHeight);
			var bodies = Main.space.bodiesInAABB(checkAABB);
			if (bodies.length > 0) {
				continue;
			} else {
				temple = new Temple(this, pos);
				break;
			}
		}
	}
	
	public function update(e=null) {
		if (!started || (!players[0].alive && tick - players[0].deathTick > 30)) return;
		
		space.step(1 / 60.0, 5, 5);
		for (player in players) player.update();
		for (npc in npcs) npc.update();
		for (projectile in projectiles) projectile.update();
		tick++;
	}
	
	public function keyDown(e: KeyboardEvent) {
		var code = getKeyCode(e);
		switch(code) {
			case Keyboard.W: players[0].up = true;
			case Keyboard.S: players[0].down = true;
			case Keyboard.A: players[0].left = true;
			case Keyboard.D: players[0].right = true;
			case Keyboard.SPACE: players[0].startConvert();
			case Keyboard.R: reset();
			case Keyboard.K: players[0].kill();
			
			//Alternate controls:
			case Keyboard.UP: players[0].up = true;
			case Keyboard.DOWN: players[0].down = true;
			case Keyboard.LEFT: players[0].left = true;
			case Keyboard.RIGHT: players[0].right = true;
			case Keyboard.CONTROL: players[0].startConvert();
		}
	}
	
	public function keyUp(e: KeyboardEvent) {
		var code = getKeyCode(e);
		switch(code) {
			case Keyboard.W: players[0].up = false;
			case Keyboard.S: players[0].down = false;
			case Keyboard.A: players[0].left = false;
			case Keyboard.D: players[0].right = false;			
			case Keyboard.SPACE: players[0].endConvert();
			
			//Alternate controls:
			case Keyboard.UP: players[0].up = false;
			case Keyboard.DOWN: players[0].down = false;
			case Keyboard.LEFT: players[0].left = false;
			case Keyboard.RIGHT: players[0].right = false;
			case Keyboard.CONTROL: players[0].endConvert();
		}
	}
	
	public function click(e: MouseEvent) {
		if (!started) {
			started = true;
			removeChild(startedOverlay);
			removeChild(startLabel);
			startLabel = null;
			startedOverlay = null;
		}
		var target = new Vec2(e.stageX, e.stageY);
		players[0].fireAt(target);
		//players[0].findPath(target);
	}
	
	static public function main() {
		var stage = Lib.current.stage;
		stage.scaleMode = nme.display.StageScaleMode.SHOW_ALL;
		stage.align = nme.display.StageAlign.TOP_LEFT;
		
		Lib.current.addChild(new Main());
	}
	
	static function getKeyCode(key) {
		var code = key.keyCode;
		#if (cpp || neko)
		if (code >= 97 && code <= 122) code -= 32;
		#end
		return code;
	}
}
