package org.norgg.hopesington;

import nape.callbacks.InteractionType;
import nape.geom.AABB;
import nape.geom.Vec2;
import nape.shape.Polygon;

import haxe.FastList;

import nme.display.Sprite;
import nme.display.DisplayObjectContainer;
import nme.Assets;

using Std;

class NPC extends Person {
	var pathTimer = 0;

	public var angry = false;
	
	var target: Entity;
	
	public static var calmSpeed = 130;
	public static var angrySpeed = 250;
	
	var exclaimTimer = 60;
	
	public function new(parent: DisplayObjectContainer, pos: Vec2) {
		texture = Assets.getBitmapData("img/npc.png");

		super(parent, pos);
	
		pathTimer = Std.int(Math.random() * 50);
		shootTimer = Std.int(Math.random() * 50);
		body.userData.npc = this;
		speed = calmSpeed;
	}
	
	override public function update() {
		super.update();
		if (alive) {
			pathTimer--;
			
			if (angry && exclaimTimer >= 0) {
				exclaimTimer--;
				if (exclaimTimer <= 0) {
					redraw();
				}
			}

			if (angry || converted) {
				shootTimer--;
				if (shootTimer <= 0) {
					shootTimer = 30 + Std.int(Math.random() * 50);
					
					if (angry) {
						if (target == null || !target.is(Person)) {
							target = Main.players[0];
						}
						
						var person: Person = cast(target, Person);
						if (person.alive) {
							fireAt(target.body.worldCOM);
						} else {
							target = null;
							clearPath();
						}
					} else if (converted) {
						if (target.is(NPC)) {
							if (cast(target, NPC).alive) {
								fireAt(target.body.worldCOM);
							} else {
								clearPath();
							}
						}
					}
				}
			}
			
			if (path == null || path.isEmpty()) {
				if (pathTimer <= 0) {
					target = null;
					if (angry) {
						var minDist = Math.POSITIVE_INFINITY;
						if (Main.players[0].alive) {
							target = Main.players[0];
							minDist = Main.players[0].body.worldCOM.sub(body.worldCOM).length;
						}
						//Target converted person if closer.
						for (npc in Main.npcs) {
							if (npc.converted && npc.alive) {
								var dist = npc.body.worldCOM.sub(body.worldCOM).length;
								if (dist < minDist) {
									target = npc;
									minDist = dist;
								}
							}
						}
						
						//Calm down if no one left to kill.
						if (target == null) {
							angry = false;
						}
					} else {
						var minDist = Math.POSITIVE_INFINITY;
						
						if (hasLog) {
							//If converted, defend the player, then try to build the temple.
							if (converted) {
								//Target nearest angry person.
								var minDist = Math.POSITIVE_INFINITY;
								for (npc in Main.npcs) {
									if (npc.angry && npc.alive) {
										var dist = npc.body.worldCOM.sub(body.worldCOM).length;
										if (dist < minDist) {
											target = npc;
											minDist = dist;
										}
									}
								}
								
								//If no angry people, build the temple.
								if (target == null && Main.temple != null && Main.temple.complete < 1) {
									target = Main.temple;
								}
								
								//If temple comlete, target nearest NPC
								if (Main.temple != null && Main.temple.complete >= 1) {
									for (npc in Main.npcs) {
										if (npc.alive && !npc.converted) {
											var dist = npc.body.worldCOM.sub(body.worldCOM).length;
											if (dist < minDist) {
												target = npc;
												minDist = dist;
											}
										}
									}
								}
							} else {
							//Otherwise build a house.
								var houseSet = Main.goodHouses;
								for (house in houseSet) {
									if (house.complete < 1) {
										var dist = house.body.worldCOM.sub(this.body.position).length;
										if (dist < minDist) {
											minDist = dist;
											target = house;
										}
									}
								}
							}
						} else {
							for (tree in Main.trees) {
								var dist = tree.body.worldCOM.sub(this.body.position).length;
								if (dist < minDist) {
									minDist = dist;
									target = tree;
								}
							}
						}
					}
					
					//If we still don't have a target and NPC is converted, follow the player.
					if (target == null && converted) {
						target = Main.players[0];
					}
					
					if (target != null) {
						findPath(target);
						if (path == null || path.isEmpty()) {
							//Couldn't find a path, move randomly, probably stuck.
							body.applyImpulse(Vec2.weak(Math.random() - .5, Math.random() - .5).muleq(10));
						}
					}
					

					
					pathTimer = 25 + Std.int(Math.random() * 50);
				}
			}
			
			if (path != null && !path.isEmpty()) {
				var dest = path.first().vec2();
				
				body.applyImpulse(dest.sub(body.worldCOM).muleq(10));
				
				if (body.worldCOM.sub(dest).length < 5) {
					path.pop();
					
					if (path.isEmpty() && target.body.worldCOM.sub(body.position).length < 30) {
						if (target.is(Tree)) {
							if (cast(target, Tree).harvest()) {
								getLog();
							}
						} else if (target.is(Building)) {
							dropLog();
							cast(target, Building).construct(this);
						}
					}
				}
			} 
			
			for (otherBody in body.interactingBodies(InteractionType.COLLISION, 1)) {
				if (!hasLog && otherBody.userData.tree != null) {
					if (otherBody.userData.tree.harvest()) {
						getLog();
					}
				}
				
				if (hasLog && otherBody.userData.building != null) {
					dropLog();
					otherBody.userData.building.construct(this);
				}
				
				body.applyImpulse(Vec2.weak(Math.random() - .5, Math.random() - .5).muleq(5));
				clearPath();
			}
			
			if (body.velocity.length > speed) {
				body.velocity.normalise().muleq(speed);
			}
		}
		body.velocity.muleq(0.7);
	}
	
	function clearPath() { path = new FastList<PathStep>(); }
	
	public function rage() {
		if (!alive || angry || converted || age < 500) return;
		angry = true;
		texture = Assets.getBitmapData("img/npc_angry.png");
		redraw();
		
		graphics.lineStyle(2, 0xff0000, 1);

		graphics.moveTo(Person.w / 2, -10);
		graphics.lineTo(Person.w / 2, -11);
		
		graphics.moveTo(Person.w / 2, -14);
		graphics.lineTo(Person.w / 2, -17);
		exclaimTimer = 120;
		
		speed = angrySpeed;
	}
	
	public function calm() {
		if (!alive || !angry) return;
		angry = false;
		
		if (hasLog) {
			texture = Assets.getBitmapData("img/npc_log.png");
		} else {
			texture = Assets.getBitmapData("img/npc.png");
		}
		redraw();
		speed = calmSpeed;
	}
	
	public function convert() {
		converted = true;
		clearPath();
		if (hasLog) {
			texture = Assets.getBitmapData("img/npc_converted_log.png");
		} else {
			texture = Assets.getBitmapData("img/npc_converted.png");
		}
		redraw();
		speed = angrySpeed;
	}
	
	public function getLog() {
		hasLog = true;
		if (converted) {
			texture = Assets.getBitmapData("img/npc_converted_log.png");
			
			var checkRange = 100;
			checkAABB.x = x - checkRange / 2;
			checkAABB.y = y - checkRange / 2;
			checkAABB.width = checkRange;
			checkAABB.height = checkRange;
			var bodies = Main.space.bodiesInAABB(checkAABB);
			for (body in bodies) {
				if (body.userData.npc != null) {
					var npc: NPC = body.userData.npc;
					if (npc.alive && npc != this && !npc.converted) {
						fireAt(npc.body.worldCOM);
						break;
					}
				}
			}
		} else {
			texture = Assets.getBitmapData("img/npc_log.png");
		}
		redraw();
	}
	
	public function dropLog() {
		hasLog = false;
		if (converted) {
			texture = Assets.getBitmapData("img/npc_converted.png");
		} else {
			texture = Assets.getBitmapData("img/npc.png");
		}
		redraw();
	}
	
	override public function kill(with: Projectile = null) {
		texture = Assets.getBitmapData("img/npc_dead.png");
		super.kill(with);
		
		if (with != null && with.owner.converted) {
			//If owner was current player or a converted NPC, make nearby NPCs angry
			var checkRange = 300;
			checkAABB.x = x - checkRange / 2;
			checkAABB.y = y - checkRange / 2;
			checkAABB.width = checkRange;
			checkAABB.height = checkRange;
			var bodies = Main.space.bodiesInAABB(checkAABB);
			for (body in bodies) {
				if (body.userData.npc != null) {
					if (body.userData.npc.alive) {
						body.userData.npc.rage();
					}
				}
			}
		}
	}
}