package org.norgg.hopesington;
import haxe.FastList;
import nape.dynamics.InteractionGroup;
import nape.geom.AABB;
import nape.phys.Body;
import nape.phys.BodyType;
import nape.shape.Polygon;
import nape.geom.Vec2;
import nape.shape.Shape;
import nme.display.BitmapData;

import nme.Assets;
import nme.display.DisplayObjectContainer;
import nme.display.Sprite;

class Person extends Entity {
	var texture: BitmapData;

	public static inline var w = 7;
	public static inline var h = 11;
	
	var speed = 150.0;
	
	var shootTimer = 0;
	
	public var age = 20;
	
	public var alive = true;
	
	public var group: InteractionGroup;
	public var projectileGroup: InteractionGroup;
	
	public var path: FastList<PathStep>;
	public var toSearch: FastList<PathStep>;
	public var searched: FastList<PathStep>;
	
	var shape: Shape;
	
	public var converted = false;
	public var hasLog = false;
		
	var checkAABB: AABB;
	
	public function new(parent: DisplayObjectContainer, pos: Vec2) {
		super();
		age = 100;
		body = new Body();
		
		shape = new Polygon(Polygon.rect(0, 0, w, h));
		body.shapes.add(shape);
		
		body.position = pos;
		body.type = BodyType.DYNAMIC;
		body.space = Main.space;
		body.allowRotation = false;
		
		body.userData.person = this;
		body.userData.blocks = true;

		group = new InteractionGroup(true);
		
		projectileGroup = new InteractionGroup(false);
		projectileGroup.group = group;
		
		body.group = group;
		x = pos.x;
		y = pos.y;

		redraw();
		
		checkAABB = new AABB();
		
		shootTimer = Std.int(Math.random() * 50);
		
		parent.addChild(this);
	}
	
	public function redraw() {
		graphics.clear();
		graphics.beginBitmapFill(texture, null, false);
		graphics.drawRect(0, 0, w, h);
	}
	
	public function update() {
		
		if (age <= 501 && alive) {
			if (age % 10 == 1) {
				body.shapes.remove(shape);
				shape = new Polygon(Polygon.rect(0, 0, Person.w * (age / 501), Person.h * (age / 501)));
				body.shapes.add(shape);
				scaleX = scaleY = age / 501;
			}
		} else {
			scaleX = scaleY = 1;
		}
		
		
		if (age > 8000) {
			kill();
		}
		
		x = Std.int(body.position.x);
		y = Std.int(body.position.y);
		rotation = 360 * body.rotation / (Math.PI * 2);
		
		if (alive) age++;
	}
	
	public function kill(with: Projectile = null) {
		if (!alive) return;

		body.velocity.muleq(0);
		alive = false;
		if (Math.random() > 0.5) {
			body.rotation = Math.PI / 2;
		} else {
			body.rotation = -Math.PI / 2;
		}
		Assets.getSound("snd/death.wav").play();
		redraw();
	}
	
	public function fireAt(target: Vec2) {
		if (!alive) return;
		
		var pos = body.position.add(new Vec2(Person.w/2, Person.h/3));
		var vel = target.sub(pos).normalise().muleq(200).add(body.velocity);
		var arrow = new Arrow(parent, pos, vel, projectileGroup, this);
	}
	
	public function findPath(target: Entity) {
		path = new FastList<PathStep>();
		toSearch = new FastList<PathStep>();
		searched = new FastList<PathStep>();
		
		var currentStep = new PathStep(Std.int(x+w/2), Std.int(y+h/2));
		toSearch.add(currentStep);
		
		var iters = 0;
		
		var g = Main.m.graphics;
		
		var searchStep = 10;
		
		var targetPos = target.body.worldCOM;
		
		while (currentStep.distTo(targetPos) > 10 && iters < 100) {
			iters++;
			//Get item closest to destination.
			var minDist = Math.POSITIVE_INFINITY;
			
			g.moveTo(currentStep.x, currentStep.y);
			g.lineStyle(1, 0, 0.1);
			
			currentStep = null;
			for (step in toSearch) {
				var dist = step.distTo(targetPos);
				//trace(dist);
				if (dist < minDist) {
					currentStep = step;
					minDist = dist;
				}
			}
			if (currentStep == null) {
				return;
			}
			
			//g.lineTo(currentStep.x, currentStep.y);

			toSearch.remove(currentStep);
			searched.add(currentStep);
			
			//trace("Checking " + currentStep.x + "," + currentStep.y);
			
			var newX = currentStep.x - searchStep;
			while (newX <= currentStep.x + searchStep) {
				var newY = currentStep.y - searchStep;
				while (newY <= currentStep.y + searchStep) {
					if (!(newX == currentStep.x && newY == currentStep.y)) {
						var newStep = new PathStep(newX, newY);
						//trace("Neighbor: " + newStep.x + "," + newStep.y);
						
						//Skip this step if we've already searched it.
						var found = false;
						for (step in searched) {
							if (step.x == newX && step.y == newY) {
								found = true;
								break;
							}
						}
						if (!found) {
							found = false;
							
							//Update this step if it's already in the toSearch set.
							for (step in toSearch) {
								if (step.x == newX && step.y == newY) {
									found = true;
									break;
								}
							}
							if (found) {
								if (newStep.costWithParent(currentStep) < newStep.cost) {
									newStep.parent = currentStep;
									newStep.updateCost();
								}
							} else {
								//Not in either set, add it to those to search.
								if (!blocked(newStep, target)) {
									newStep.parent = currentStep;
									newStep.updateCost();
									toSearch.add(newStep);
								}
							}
						}
					}
					newY += searchStep;
				}
				newX += searchStep;
			}
		}
		
		while (currentStep != null) {
			path.add(currentStep);
			currentStep = currentStep.parent;
		}
		
		g.lineStyle(0, 0);
	}
	
	public function blocked(step: PathStep, target: Entity) {
		var xClear = w*1.85;
		var yClear = h*1.85;
		
		checkAABB.x = step.x - xClear / 2;
		checkAABB.y = step.y - yClear / 2;
		checkAABB.width = xClear;
		checkAABB.height = yClear;
		var bodies = Main.space.bodiesInAABB(checkAABB);
		for (blockingBody in bodies) {
			if (blockingBody != this.body && blockingBody != target.body && blockingBody.userData.blocks) {
				return true;
			}
		}
		return false;
	}
}
