package org.norgg.hopesington;

import nape.callbacks.InteractionType;
import nme.display.DisplayObjectContainer;
import nape.geom.Vec2;
import nme.Assets;
import nme.display.Sprite;
import nme.events.Event;
import nme.geom.Matrix;
import nme.media.Sound;
import nme.media.SoundChannel;
 
class Player extends Person {

	public var left = false;
	public var right = false;
	public var up = false;
	public var down = false;
	public var converting = false;
	
	public var deathTick = 0;
	var convertSound: Sound;
	var convertSoundChannel: SoundChannel;
	
	public function new(parent: DisplayObjectContainer, pos: Vec2) {
		texture = Assets.getBitmapData("img/player.png");
		super(parent, pos);
		convertSound = Assets.getSound("snd/convert.wav");
		converted = true;
		age = 1000;
	}
	
	override public function update() {
		super.update();
		
		if (age < 1) {
			age = 1;
		}
		
		if (age > 5000) {
			speed = 150 - ((age-5000) / 100);
		}
		
		body.velocity.muleq(0.7);
		
		if (alive) {
			Main.ageLabel.text = Std.int(age / 100) + "";
			
			if (up) body.applyImpulse(new Vec2(0, -speed));
			if (down) body.applyImpulse(new Vec2(0, speed));

			if (left) body.applyImpulse(new Vec2(-speed, 0));
			if (right) body.applyImpulse(new Vec2(speed, 0));
			
			if (body.velocity.length > speed) {
				body.velocity.normalise().muleq(speed);
			}
			
			if (shootTimer > 0) shootTimer--;
		} else {
			var g = Main.m.graphics;
			g.beginFill(0x000000, 0.03);
			g.drawRect(0, 0, stage.stageWidth, stage.stageHeight);
		}

		for (otherBody in body.interactingBodies(InteractionType.COLLISION, 1)) {
			if (!hasLog && otherBody.userData.tree != null) {
				if (otherBody.userData.tree.harvest()) {
					getLog();
				}
			}
			
			if (hasLog && otherBody.userData.building != null) {
				if (otherBody.userData.building.complete < 1) {
					dropLog();
					otherBody.userData.building.construct(this);
				}
			}
		}
	}
	
	override public function fireAt(target:Vec2) {
		if (shootTimer <= 0) {
			super.fireAt(target);
			shootTimer = 30;
			dropLog();
		}
	}
	
	override public function kill(with: Projectile = null) {
		if (!alive) return;
		super.kill(with );
		
		Main.ageLabel.width = 140;
		Main.ageLabel.text = "DEAD";
		
		
		var deaths = 0;
		for (npc in Main.npcs) {
			if (!npc.alive && npc.age < 8000) {
				deaths++;
			}
		}
		
		Main.yrsLabel.x = 4;
		Main.yrsLabel.y = 50;
		Main.yrsLabel.text = "Age: " + Std.int(age / 100) + "yrs\nCost: \nGlory: " + Std.int(Main.temple.complete * 100) + "%\nR to restart";

		var texture = Assets.getBitmapData("img/npc_dead.png");
		var y = 93;
		var x = 80;
		for (i in 0...deaths) {
			var s = new Sprite();
			Main.m.addChild(s);

			var g = s.graphics;
			g.lineStyle(0, 0, 0);
			g.beginBitmapFill(texture);
			g.drawRect(0, 0, 7, 11);

			s.x = x;
			s.y = y;
			s.rotation = 90;
			x += 14;
		}
		
		deathTick = Main.tick;
		
		for (npc in Main.npcs) {
			npc.calm();
		}
	}
	
	public function startConvert() {
		if (!alive || converting) return;
		converting = true;
		var checkRange = 100*scaleX;
		graphics.lineStyle(1, 0xff0000, 0.1);
		graphics.beginFill(0, 0);
		graphics.drawCircle(body.localCOM.x, body.localCOM.y, 50);
		convertSoundChannel = convertSound.play();
		convertSoundChannel.addEventListener(Event.COMPLETE, endConvert);
	}
	
	public function endConvert(e=null) {
		redraw();
		if (convertSoundChannel != null) {
			convertSoundChannel.stop();
		}
		if (!alive || !converting) return;
		converting = false;
		
		var checkRange = 100;
		checkAABB.x = body.worldCOM.x - checkRange / 2;
		checkAABB.y = body.worldCOM.y - checkRange / 2;
		checkAABB.width = checkRange;
		checkAABB.height = checkRange;
		
		var firstConverted = false;

		var bodies = Main.space.bodiesInAABB(checkAABB);
		for (body in bodies) {
			if (body.worldCOM.sub(this.body.position).length > checkRange/2) continue;
			if (body.userData.npc != null) {
				var npc: NPC = body.userData.npc;
				if (npc.angry || npc.converted || !npc.alive) continue;
				
				if (firstConverted) {
					npc.rage();
				} else {
					npc.convert();
					npc.age += 100;
					//age -= 250;
					firstConverted = true;
				}
			}
			Assets.getSound("snd/convert_success.wav").play();
		}
	}
	
	public function getLog() {
		hasLog = true;
		texture = Assets.getBitmapData("img/player_log.png");
		redraw();
	}
	
	public function dropLog() {
		hasLog = false;
		texture = Assets.getBitmapData("img/player.png");
		redraw();
	}
}